-- ######################################################
-- ## Script by Ollicious,  bowdown@gmx.net            ##
-- ## V 1.0, 2015/09/15                                ##
-- ## Dynamic design via initial values and functions  ##
-- ## Some of the Widgets and BMPs based on work by    ##
-- ## Sockeye                                          ##
-- ## -------------------------------------------------##
-- ## edit by Dirk Wesenberg, 2017/09/28               ##
-- ## DeWe (http://www.kopterforum.at)				   ##
-- ## - Changes for OpenTX 2.2.0                       ##
-- ## - add GPS Radar, Source from 					   ##
-- ##  https://github.com/Clooney82/MavLink_FrSkySPort ##
-- ##									    		   ##
-- ##  GNU GENERAL PUBLIC LICENSE; Vers 3, 29 June 2007##     
-- ######################################################

-- #################### Definitions #####################

------------------------------------------------------------
-- Widget definition:                                     --
-- The first line is commented and does the same as the   --
-- multiple lines below                                   --
-- Decide which format you prefer and then edit the array -- 
-- accordingly to how you want the widgets placed.        --
--                                                        --
-- Widgets in V1.1                                        --
-- Voltage: "battery", to be put in column alone          --
-- RSSI: "rssi", to be put in column alone                --
-- Consumption: "capacity", to be put in column alone     --
-- Flightmode: "fm"                                       --
-- GPS Status: "gps"                                      --
-- Timer: "timer"                                         --
-- Distance: "dist"                                       --
-- Height: "height"                                       --
-- Speed: "speed"                                         --
-- Heading: "heading"                                       --
-- Homepoint: "hp"                                        --
-- Logging: "logging" 									  --
-- Radar: "radar"                                    	  --
-- Radar bentigt 2 Felder : "dummy"					  --
-- Totalcurrent : "current"								  --
-- HDOP for APM / Teensy added	: "hdop"			 	  --
-- Armed Bit for APM / Teensy added						  --
-- Calc one Cell Voltage for APM / Teensy added			  --
--  ****************************************************  --
-- If you add own widgets, you need to add en entry to    --
-- function callWidget(name, xCoord, yCoord).             --
--                                                        --
--  The RadarWidget need the "fm" Widget to trigger the   --
--  "Arming" for the Homepoint of the Pilot               --
--  Also the "dummy" Widget is needed, for that the Radar --
--  received enough Space on the screen					  --
--  like {"fm","radar","dummy"}                           --
-- 														  --
--  Please keep care with 5S or 7S Lipos!                 --
--  => change battype manual in the init section          --
-- 	Highvolt Lipos are not supported in this moment       --
--                                                        -- 
------------------------------------------------------------

-- local widgetDefinition = {{"battery"},{"fm", "gps","timer"},{"dist", "height"},{"hp","logging","heading"},{"rssi"}}
-- local widgetDefinition = {{"battery"},{"hp", "gps","timer"},{"current", "Altheight", "speed"},{"fm","radar","dummy"},{"rssi"}}
  local widgetDefinition = {{"battery"},{"hp", "gps","timer"},{"hdop", "Altheight", "speed"},{"fm","radar","dummy"},{"rssi"}}

-- local widgetDefinition = {}
-- widgetDefinition[1] = {"battery"}
-- widgetDefinition[2] = {"capacity"}
-- widgetDefinition[3] = {"rssi"}
-- widgetDefinition[4] = {"fm", "gps","timer"}
-- widgetDefinition[5] = {"dist", "height", "speed"}
-- widgetDefinition[6] = {"hp","radar","dummy"}

-- Edit to set the width of single widget columns
local battype 				-- dont Support 5s&7s Battery, its Danger to Detect: if you have an Empty 8s its lock like an 7s...
							-- set fix to 5S or 7S change "battype = 0" to "battype = 5" in the init section 
local FC_type				-- set this Type to "BF" (BetaFlight), "INAV", "APM" means via Teensy,  .... in the init section
local xOffsetSingle = 35 
-- Leave as is!
local displayWidth = 212
local displayHeight = 64
local xOffset
local xOffsetMulti
local numSingleCols
local numMultiCols
local myMaxV
local myMinV
local myNumSat                                
local myQualSat                                 
local gpsCurrentLocation
local gpsId 
local gpsLatLon
local HomeAlt 
local set_prearm
local prearmheading
local pilotlat
local pilotlon
local gpsLatLon = {}
	local LocationLat = 0
	local LocationLon = 0
local CenterXcolArrow
local CenterYrowArrow
local sinCorr
local cosCorr
local offsetX
local offsetY
local radarxtmp
local divtmp
local upppp
local divvv
local last_pos_lat
local last_pos_lon
local oldTime
local Time
local lastsaynbattpercent
local compare
local FC_type
local totalbatteryComsum
local flightModeNumber
local hdop

-- ################### Functions ########################
local function APM_RPM ()

	local messageWord = getValue("RPM")
	local extensionCommand = 0
	
		if messageWord ~= 0 then
			extensionCommand = bit32.rshift(messageWord, 12)
			extensionCommand = bit32.band(extensionCommand, 15)
			
			local extensionData = bit32.band(messageWord, 4095)
			
			if extensionCommand == 4 then
				if (extensionData ~= nil)  then
						hdop = extensionData
				end
			
			end
			
			if extensionCommand == 6 then                                   
						if (extensionData ~= nil)  then
						-- myNumSat =myNumSat +1
							myNumSat = extensionData
							myQualSat = extensionData
						end
			end
			
			
			if extensionCommand == 7 then                                    
						if (extensionData ~= nil)  then
							set_prearm = true
						end 
			end
			
			if extensionCommand == 8 then
					flightModeNumber = extensionData				-- verkrzt
						if flightModeNumber ~= nil then
							flightModeNumber = flightModeNumber + 1
									if flightModeNumber < 1 or flightModeNumber > 17 then
										flightModeNumber = 18 			-- "---"
									end	
						end
			end
		end
end



--------------
-- Rounding --
--------------
local function rnd(v,d)
	if d then
	 return math.floor((v*10^d)+0.5)/(10^d)
	else
	 return math.floor(v+0.5)
	end
end

------------------
-- Telemetry ID --
------------------
local function getTelemetryId(name)
	field = getFieldInfo(name)
	if field then
	  return field.id
	else
	  return -1
	end
end
 
---------------
-- Get Value --
--------------- 
local function getValueOrDefault(value)
	local tmp = getValue(value)
	
	if tmp == nil then
		return 0
	end
	
	return tmp
end

--------------------------------------------------------------------------------
--  Lipo Cell Dection 
--------------------------------------------------------------------------------
local function BatteryCalcCellVoltageAndTyp()
   
   local comp_1
   local comp_2
   local comp_3
   
    if (battype==0) then
		if getValueOrDefault("RSSI") > 55 then
		
			comp_1 = getValueOrDefault("VFAS") 
			comp_2 = getValueOrDefault("VFAS")
			
				if comp_1 == comp_2 then
					compare = compare + 1
				end
 
				if compare > 50 then
								  if math.floor(comp_1/4.37) > battype and comp_1 < 4.37*8 then 
									 battype=math.ceil(comp_1/4.37)
									 if battype==7 then battype=8 end --dont Support 5s&7s Battery, its Danger to Detect: if you have an Empty 8s its lock like an 7s...
									 if battype==5 then battype=6 end 
								   end
					  
								  if battype > 0 then  
									 myMaxV = math.floor(42 * battype)
									 myMinV = math.floor(34 * battype) -- 0% wren 3,4Volt ... 3,3V sind schon sehr grenzwertig da hier die Chemie zerstoert wird
								  end
				end				
		end
	end
end
--------------------------------------------------------------------------------
--  Lipo Cell Dection 
-- get from https://github.com/Lupinixx/LuaPilot_Taranis_Telemetry_Cleanflight_INAV
-- All credits go to the original creators of the script. 
--------------------------------------------------------------------------------	

local function SayBattPercent(battpercent)  

  if (battpercent < (lastsaynbattpercent-9)) then --only say in 10 % steps
       
    Time[6] = Time[6] + (getTime() - oldTime[6]) 
        
    if Time[6]> 300 then --and only say if battpercent 10 % below for more than 3sec
      lastsaynbattpercent=(rnd(battpercent*0.1)*10)
      Time[6] = 0
      playNumber(rnd(lastsaynbattpercent), 13, 0)
	  if ((lastsaynbattpercent <= 30) and (lastsaynbattpercent > 20)) then 
        playFile("/SCRIPTS/TELEMETRY/WAV/Akksch.wav") 
      end
      if lastsaynbattpercent <= 20 then 
        playFile("/SCRIPTS/TELEMETRY/WAV/AkkAla.wav") 
      end
    end
    
    oldTime[6] = getTime() 
  
  else    
    Time[6] = 0
    oldTime[6] = getTime() 
  end
  
end

-- #################### Widgets #########################
----------------------------------------
-- Current  	MULTI Row Col Widget --
----------------------------------------
   local function totcurrWidget(xCoord, yCoord)
   
	
      Time[1] = Time[1] + (getTime() - oldTime[1])
      if Time[1] >=20 then --200 ms
        totalbatteryComsum  = totalbatteryComsum + ( getValueOrDefault("Curr") * (Time[1]/360))
        Time[1] = 0
		 oldTime[1] = getTime() 
      end
			 -- oldTime[1] = getTime()  -- das das Script schneller ist, kommen wir nie in den if Zweig
	  
		lcd.drawPixmap(xCoord + 2, yCoord + 2, "/SCRIPTS/TELEMETRY/OLIME/BattA.bmp")
	    lcd.drawNumber(xCoord + 20, yCoord + 5,totalbatteryComsum , LEFT)
		lcd.drawText(xCoord + 23, yCoord + 13, "mAh", SMLSIZE)
	  
    end
	
----------------------------------------
-- HDOP;     SINGLE Row Col Widget --
----------------------------------------
local function hdopWidget(xCoord, yCoord)
	lcd.drawPixmap(xCoord + 1, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/hdop.bmp") 	
		  if hdop ~= nil then

				local lhdop = hdop / 10
				  if lhdop >= 99 then
					lcd.drawText(xCoord + 24, yCoord+2, ">", SMLSIZE)  
					lcd.drawNumber(xCoord + 28, yCoord+2, 99, MIDSIZE)
				  else
					lcd.drawNumber(xCoord + 24, yCoord+2, lhdop*10, PREC1 + MIDSIZE)
				  end
			end
end	
----------------------------------------
-- Battery;     SINGLE Row Col Widget --
----------------------------------------
local function batteryWidget(xCoord, yCoord)

	
	
	if (battype==0) then
		BatteryCalcCellVoltageAndTyp()
	end
	
	local myCurrent = 0
	local myPercent = 0
	 local mincell = 0
	
		if (((myMinV == 0) or (myMaxV == 0)) and (battype > 0))then
					myMaxV = math.floor(42 * battype)
					myMinV = math.floor(34 * battype)						-- 0% wren 3,4Volt ... 3,3V sind schon sehr grenzwertig da hier die Chemie zerstoert wird
		end
		
		if battype > 0 then
		   myCurrent = getValueOrDefault("VFAS") * 10                            
		   local myRangeV = myMaxV - myMinV                           -- Nutzspannung berechnen
		   local myAvailV = myCurrent - myMinV                        -- Restliche Nutzspannung berechnen
		   myPercent = math.floor(myAvailV / myRangeV * 100)    	  -- Restliche Nutzspannung in Prozent
					if myPercent < 1 then
						myPercent = 0
					end
						SayBattPercent(myPercent)
		 end  
		   if ((myAvailV == nil) or (myAvailV < 0 ))then myAvailV = 0 end                      -- Fehlerhafte Werte abfangen
		   if ((myPercent == nil) or (myPercent < 0)) then myPercent = 0 end                   -- Fehlerhafte Werte abfangen
		   if myPercent > 100 then myPercent = 100 end                -- Fehlerhafte Werte abfangen
		   
		   local myPxHeight = math.floor(myPercent * 0.37)            -- Bentigte Zeilenhhe ermitteln
		   local myPxY = 11 + 37 - myPxHeight

		   lcd.drawPixmap(xCoord + 1, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/battery.bmp")           -- Leere Batterie anzeigen

		   if myPercent > 0 then
				lcd.drawFilledRectangle(xCoord + 6, myPxY, xCoord + 21, myPxHeight, FILL_WHITE ) 
		   end

			 if (getValueOrDefault("Cell") > 0) then                     
				mincell = getValueOrDefault("Cell") * 10
			 elseif (getValueOrDefault("A4") > 0 ) then
				mincell = getValueOrDefault("A4") * 10
			 elseif ((battype > 0) and (myCurrent > 0)) then
				mincell = myCurrent / battype
			 end
			 
		   if mincell < 35 then
				lcd.drawNumber(xCoord + 8,yCoord + 13, mincell ,PREC1 + LEFT + BLINK + SMLSIZE)
		   else
				lcd.drawNumber(xCoord + 8,yCoord + 13, mincell ,PREC1 + LEFT + SMLSIZE)
		   end
		   lcd.drawText(lcd.getLastPos(), yCoord + 13, "V", SMLSIZE)
		   
		  
		   if (myCurrent > myMaxV) or (myCurrent < myMinV) then
				lcd.drawNumber(xCoord + 10,yCoord + 55, myCurrent ,PREC1 + LEFT + BLINK+ SMLSIZE)
		   else
				lcd.drawNumber(xCoord + 7,yCoord + 55, myCurrent ,PREC1 + LEFT + SMLSIZE)
		   end
		   lcd.drawText(lcd.getLastPos(), yCoord + 55, "V", SMLSIZE)
		   


end

---------------------------------------- 
-- RSSI;        SINGLE Row Col Widget --
----------------------------------------
local function rssiWidget(xCoord,yCoord)

  local percent = 0
  
  if getValueOrDefault("RSSI") > 38 then
   percent = ((math.log(getValueOrDefault("RSSI")-28, 10)-1)/(math.log(72, 10)-1))*100
   else
   percent = 0
  end

   if getValueOrDefault("RSSI") < 38 then
     lcd.drawNumber(xCoord + 8, yCoord + 54, getValueOrDefault("RSSI"), LEFT + INVERS + BLINK)
     lcd.drawText(lcd.getLastPos(), yCoord + 54, "db", INVERS + BLINK)
   else
      lcd.drawNumber(xCoord + 8, yCoord + 54, getValueOrDefault("RSSI"), LEFT)
      lcd.drawText(lcd.getLastPos(), yCoord + 54, "db", 0)
   end
   
   
  if percent > 90 then
   	lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh10.bmp")
  elseif percent > 80 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh09.bmp")
  elseif percent > 70 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh08.bmp")
  elseif percent > 60 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh07.bmp")
  elseif percent > 50 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh06.bmp")
	elseif percent > 40 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh05.bmp")
  elseif percent > 30 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh04.bmp")
  elseif percent > 20 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh03.bmp")
  elseif percent > 10 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh02.bmp")
  elseif percent > 0 then
		lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh01.bmp")
  else
    lcd.drawPixmap(xCoord + 4, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/RSSIh00.bmp")
  end

end  


------------------------------------------
-- Distance;       MULTI Row Col Widget --
------------------------------------------
local function distWidget(xCoord, yCoord)

   local dist = getValueOrDefault("Dist")
   
   if simModeOn == 1 then
      dist = tdist
   end
   
   lcd.drawPixmap(xCoord + 2, yCoord + 2, "/SCRIPTS/TELEMETRY/OLIME/dist.bmp")
   lcd.drawNumber(xCoord + 18, yCoord + 7, dist, LEFT)
   lcd.drawText(lcd.getLastPos(), yCoord+ 7, "m", 0)
   
end
   
------------------------------------------
--  ALT Height;        MULTI Row Col Widget --
------------------------------------------
local function AltheightWidget(xCoord, yCoord)

		lcd.drawPixmap(xCoord + 1, yCoord + 2, "/SCRIPTS/TELEMETRY/OLIME/hgt.bmp")
		flight_height = getValueOrDefault("Alt")										-- Hoehe von ALT
		lcd.drawNumber(xCoord + 18, yCoord + 7, flight_height, LEFT)
		lcd.drawText(lcd.getLastPos(), yCoord + 7, "m", 0)
end
   
------------------------------------------
--  Height;        MULTI Row Col Widget --
------------------------------------------
local function GPSheightWidget(xCoord, yCoord)

	local height = getValueOrDefault("GAlt")									-- Hoehe vom GPS

			   flight_height = height - HomeAlt
					if  HomeAlt > 0 then
						lcd.drawPixmap(xCoord + 1, yCoord + 2, "/SCRIPTS/TELEMETRY/OLIME/hgt.bmp")
					else 
						lcd.drawPixmap(xCoord + 1, yCoord + 2, "/SCRIPTS/TELEMETRY/OLIME/hgtNN.bmp")
					end
					
	lcd.drawNumber(xCoord + 18, yCoord + 7, flight_height, LEFT)
	lcd.drawText(lcd.getLastPos(), yCoord + 7, "m", 0)
end

------------------------------------------
--  Speed;        MULTI Row Col Widget --
------------------------------------------
local function speedWidget(xCoord, yCoord)

   local speed = getValueOrDefault("GSpd")
   
   lcd.drawPixmap(xCoord + 1, yCoord + 2, "/SCRIPTS/TELEMETRY/OLIME/speed.bmp")
   lcd.drawNumber(xCoord + 18, yCoord + 7, speed * 1.852, LEFT)
   lcd.drawText(lcd.getLastPos(), yCoord + 7, "kmh", SMLSIZE)
end

------------------------------------------
--  Heading;       MULTI Row Col Widget --
------------------------------------------
local function headingWidget(xCoord, yCoord)

   local heading = getValueOrDefault("Hdg") 
   
   lcd.drawPixmap(xCoord + 1, yCoord + 2, "/SCRIPTS/TELEMETRY/OLIME/compass.bmp")
   lcd.drawNumber(xCoord + 18, yCoord + 7, heading, LEFT)
   lcd.drawText(lcd.getLastPos(), yCoord + 7, "dg", 0)
end

------------------------------------------
--  Logging;        MULTI Row Col Widget --
------------------------------------------
local function loggingWidget(xCoord, yCoord)

    if getValueOrDefault(103) > 50 then                                         
      lcd.drawText(xCoord + 16, yCoord+ 3,"ON", MIDSIZE)
      lcd.drawPixmap(xCoord + 2, yCoord+ 5, "/SCRIPTS/TELEMETRY/OLIME/save.bmp")
   else
      lcd.drawText(xCoord + 16,yCoord+ 3,"OFF", MIDSIZE)
      lcd.drawPixmap(xCoord + 2, yCoord+ 5, "/SCRIPTS/TELEMETRY/OLIME/saveoff.bmp")

   end
end

------------------------------------------------------
-- Idee und Umsetzung von LuaPilot  2017 ilihack
-- Aenderungen und an das Script angepasst DeWe
------------------------------------------------------

	local function fmWidget(xCoord, yCoord) 
		lcd.drawPixmap(xCoord + 1, yCoord + 2, "/SCRIPTS/TELEMETRY/OLIME/fm.bmp")            -- Flugzeugbmp anzeigen
		
		--local FM_Tmp1 = getValueOrDefault("Tmp1")
		local ModeName = {}

				ModeName[1]= "Stab"
				ModeName[2]= "Acro"
				ModeName[3]= "AltHO"
				ModeName[4]= "Auto"
				ModeName[5]= "Guide"
				ModeName[6]= "Loit"
				ModeName[7]= "RTH"
				ModeName[8]= "Circ"
				ModeName[9]= "Inv-M"
				ModeName[10]= "Land"
				ModeName[11]= "OLoit"
				ModeName[12]= "Drift"
				ModeName[13]= "Inv-M"
				ModeName[14]= "Sport"
				ModeName[15]= "FlipF"
				ModeName[16]= "AutoT"
				ModeName[17]= "PosHO"
				ModeName[18]= "---"
				ModeName[19]= "NoTel"
				
		local convertedFlightMode = 18
						if ((flightModeNumber > 0) and (flightModeNumber < 20)) then
								convertedFlightMode = flightModeNumber		
						end
				lcd.drawText(xCoord + 18, yCoord + 7, ModeName[convertedFlightMode],0)
				--lcd.drawText(xCoord + 18, yCoord + 7, convertedFlightMode,0)
   end
  
------------------------------------------
-- Timer;          MULTI Row Col Widget --
------------------------------------------
local function timerWidget(xCoord, yCoord)

  lcd.drawPixmap(xCoord + 1, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/timer_1.bmp")
  
  local localtimer = model.getTimer(0)
			if (localtimer.value < 3600) then
				lcd.drawTimer(xCoord + 18, yCoord + 8, localtimer.value, 0)
			else
					lcd.drawTimer(xCoord + 18, yCoord + 8, localtimer.value / 60 , 0)
					lcd.drawText(lcd.getLastPos()+0,50,":",0+BLINK)
			end

 end
 
 -----------------------------------------
 -- GPS;           MULTI Row Col Widget --
 -----------------------------------------
 -- ToDo xx[0-9]xx => Hdop ???
 local function gpsWidget(xCoord,yCoord)
 


		local gpsLatLon = getValueOrDefault("GPS")
		
		if (type(gpsLatLon) == "table") then
			if ((gpsLatLon["lat"] ~= NIL) and (gpsLatLon["lat"] ~= 0 )) then
				last_pos_lat = gpsLatLon["lat"]
			end
			if ((gpsLatLon["lon"] ~= NIL) and (gpsLatLon["lat"] ~= 0 )) then
				last_pos_lon = gpsLatLon["lon"]
			end
		end
		
		-- local gpsLatLon = getValueOrDefault("GPS")
	
	if ((myNumSat == 0) and (last_pos_lon > 0) and (last_pos_lat > 0)) then  -- todo irgendwas das toogled
				lcd.drawText(xCoord +2, yCoord + 6,last_pos_lat , SMLSIZE)
				lcd.drawText(xCoord +2, yCoord + 12,last_pos_lon , SMLSIZE)
	else
		
				if myQualSat == 0 then
					lcd.drawPixmap(xCoord + 1, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/sat0.bmp")         -- kein Fix
				elseif myQualSat == 3 then																-- 1 is GPS Fix (Neu)
					lcd.drawPixmap(xCoord + 1, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/sat1.bmp")                
				elseif ((myQualSat > 3) and (myQualSat < 9)) then																--2 is GPS Home fix (Neu)
					lcd.drawPixmap(xCoord + 1, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/sat2.bmp")
				elseif myQualSat > 8 then
					lcd.drawPixmap(xCoord + 1, yCoord + 1, "/SCRIPTS/TELEMETRY/OLIME/sat3.bmp")                	-- Differenzial GPS (DGPS)  (auskommentiert)
				end				
				 
				 
				if myNumSat > 5 then
					lcd.drawPixmap(xCoord + 13, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/gps_6.bmp")               -- 6 oder mehr Sats 
				elseif myNumSat > 4 then
					lcd.drawPixmap(xCoord + 13, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/gps_5.bmp")               -- 5 Sats 
				elseif myNumSat > 3 then
					lcd.drawPixmap(xCoord + 13, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/gps_4.bmp")               -- 4 Sats
				elseif myNumSat > 2 then
					lcd.drawPixmap(xCoord + 13, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/gps_3.bmp")               -- 3 Sats
				elseif myNumSat > 1 then
					lcd.drawPixmap(xCoord + 13, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/gps_2.bmp")               -- 2 Sats
				elseif myNumSat > 0 then
					lcd.drawPixmap(xCoord + 13, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/gps_1.bmp")               -- 1 Sat
				else
					lcd.drawPixmap(xCoord + 13, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/gps_0.bmp")                 -- kein Sat
				end
				 
			   lcd.drawNumber(xCoord + 24, yCoord + 1, myNumSat,  SMLSIZE)                        -- Anzahl anzeigen
	end
 end

 -----------------------------------------
 -- Home Point;    MULTI Row Col Widget --
 -----------------------------------------
 local function hpWidget(xCoord, yCoord)
--   if myQualSat < 3 then
	 if myQualSat < 2 then								-- bei CF und BF bekommen wir lt Beschreibung von CF nur 0,1 und 2; 2 = Home Fix
      lcd.drawPixmap(xCoord + 1, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/home1.bmp")
      lcd.drawText(xCoord + 19, yCoord + 5, "?", MIDSIZE + BLINK)
   elseif myNumSat > 6 then								-- erhht auf min 7 Sat's 
      lcd.drawPixmap(xCoord + 1, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/home.bmp")
      lcd.drawPixmap(xCoord + 19, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/check.bmp")
   else
      lcd.drawPixmap(xCoord + 1, yCoord + 3, "/SCRIPTS/TELEMETRY/OLIME/home.bmp")
--      lcd.drawText(xCoord + 19, yCoord + 5, "X",  BLINK+ MIDSIZE)
      lcd.drawText(xCoord + 19, yCoord + 5, myNumSat ,  BLINK + MIDSIZE)
   end
 end
 -----------------------------------------
 -- Radar;    MULTI Row Col Widget --
 ----------------------------------------- 
	
local function dummyWidget (xCoord, yCoord)	
	-- for 2 places 
end
 local function radarWidget (xCoord, yCoord)
	
		hypdist = 0
									
		t2 = getValueOrDefault("Tmp2")														
																-- apmarmed = t2%0x02
		gpsLatLon = getValueOrDefault("GPS")
				if (type(gpsLatLon) == "table") then
					if gpsLatLon["lat"] ~= NIL then
						LocationLat = gpsLatLon["lat"]
					end
					if gpsLatLon["lon"] ~= NIL then
						LocationLon = gpsLatLon["lon"]
					end
				end
		--- if apmarmed ~=1 then -- heading/position before arming. Used for radar once armed
			if set_prearm then
					prearmheading = getValueOrDefault("Hdg")
						pilotlat = math.rad(LocationLat)
						pilotlon = math.rad(LocationLon)
					set_prearm = false
			end

			curlat = math.rad(LocationLat)
			curlon = math.rad(LocationLon)

		
		if pilotlat~=0 and curlat~=0 and pilotlon~=0 and curlon~=0 then
			z1 = math.sin(curlon - pilotlon) * math.cos(curlat)
			z2 = math.cos(pilotlat) * math.sin(curlat) - math.sin(pilotlat) * math.cos(curlat) * math.cos(curlon - pilotlon)
			
			-- use prearmheading later to rotate cordinates relative to copter.
			radarx=z1*6358364.9098634 															-- meters for x absolut to center(homeposition)
			radary=z2*6358364.9098634 															-- meters for y absolut to center(homeposition)
			
			hypdist =  math.sqrt( math.pow(math.abs(radarx),2) + math.pow(math.abs(radary),2) )
			radTmp = math.rad( prearmheading )
			radarxtmp = radarx * math.cos(radTmp) - radary * math.sin(radTmp)
			radarytmp = radarx * math.sin(radTmp) + radary * math.cos(radTmp)
			if math.abs(radarxtmp) >= math.abs(radarytmp) then 									--divtmp
				for i = 13 ,1,-1 do
					if math.abs(radarxtmp) >= upppp then
						divtmp=divvv
						break
					end
					divvv = divvv/2
					upppp = upppp/2
				end
			else
				for i = 13 ,1,-1 do
				
					if math.abs(radarytmp) >= upppp then
						divtmp=divvv
						break
					end
					divvv = divvv/2
					upppp = upppp/2
				end
			end
			
			upppp = 20480
			divvv = 2048 -- x mal teilen
			
			offsetX = radarxtmp / divtmp
			offsetY = (radarytmp / divtmp)*-1
		end
		-- Radar zeichnen
		 lcd.drawRectangle(xCoord +3 ,yCoord+0 , 41, 41, DBLSIZE)      
		 lcd.drawText(xCoord +21 ,yCoord + 16,"o",0) 		
		
			for j= xCoord +5, xCoord +43, 4 do					
				lcd.drawPoint(j, yCoord +20)
			end
			for j= yCoord +2, yCoord +39, 4 do
				lcd.drawPoint(xCoord+23, j)   
			end
		-- Distanz anzeigen
			lcd.drawNumber(xCoord +24, yCoord +36,hypdist, SMLSIZE+RIGHT)
			lcd.drawText(lcd.getLastPos(), yCoord +36, "m", SMLSIZE)
		

	CenterXcolArrow = xCoord + 23
	CenterYrowArrow = yCoord + 20
		
	local arrowLine = {
			{-4, 5, 0, -4},
			{-3, 5, 0, -3},
			{3, 5, 0, -3},
			{4, 5, 0, -4}
		}
		
		sinCorr = math.sin(math.rad(getValueOrDefault("Hdg")-prearmheading))
		cosCorr = math.cos(math.rad(getValueOrDefault("Hdg")-prearmheading))
		for index, point in pairs(arrowLine) do
			X1 = CenterXcolArrow + offsetX + math.floor(point[1] * cosCorr - point[2] * sinCorr + 0.5)
			Y1 = CenterYrowArrow + offsetY + math.floor(point[1] * sinCorr + point[2] * cosCorr + 0.5)
			X2 = CenterXcolArrow + offsetX + math.floor(point[3] * cosCorr - point[4] * sinCorr + 0.5)
			Y2 = CenterYrowArrow + offsetY + math.floor(point[3] * sinCorr + point[4] * cosCorr + 0.5)
			if X1 == X2 and Y1 == Y2 then
				lcd.drawPoint(X1, Y1, SOLID, FORCE)
			else
				lcd.drawLine (X1, Y1, X2, Y2, SOLID, FORCE)
				
			end
		end	
	end

-- ######################################################
 
-------------------------
-- Call defined Widget --
-------------------------
local function callWidget(name, xPos, yPos)
	if (xPos ~= nil and yPos ~= nil) then
		if (name == "battery") then
			batteryWidget(xPos, yPos)
		elseif (name == "dist") then
			distWidget(xPos, yPos)
		elseif (name == "Altheight") then
			AltheightWidget(xPos, yPos)
		elseif (name == "GPSheight") then
			GPSheightWidget(xPos, yPos)
		elseif (name == "fm") then
			fmWidget(xPos, yPos)
		elseif (name == "timer") then
			timerWidget(xPos, yPos)
		elseif (name == "gps") then
			gpsWidget(xPos, yPos)
		elseif (name == "rssi") then
			rssiWidget(xPos, yPos)
		elseif (name == "capacity") then
			capacityWidget(xPos, yPos)
		elseif (name == "hp") then
			hpWidget(xPos, yPos)
		elseif (name == "speed") then
			speedWidget(xPos, yPos)
		elseif (name == "logging") then
			loggingWidget(xPos, yPos)
		elseif (name == "heading") then
			headingWidget(xPos, yPos)
		elseif (name == "radar") then
			radarWidget(xPos, yPos)
		elseif (name == "dummy") then
			dummyWidget(xPos, yPos)
		elseif (name == "current") then
			totcurrWidget(xPos, yPos)
		elseif (name == "hdop") then
			hdopWidget(xPos, yPos)
		else
			return
		end
	end
end

---------------------
--   Build Grid    --
---------------------
local function buildGrid(def)

	local tempSumX = -1
	local tempSumY = -1
	
	-- ## Cols
	for i=1, # def, 1
	do
		-- make single Columns narrower
		if (# def[i] == 1) then
			xOffset = xOffsetSingle
		else
			xOffset = xOffsetMulti
		end
		
		-- Vertical Lines
		lcd.drawLine(tempSumX, -1, tempSumX, displayHeight, SOLID, GREY_DEFAULT)

		-- ## Rows
		for j=1, # def[i], 1
		do
			-- Horizontal Lines			
			lcd.drawLine(tempSumX, tempSumY, tempSumX + xOffset, tempSumY, SOLID, GREY_DEFAULT)

			
			-- Widget
			callWidget(def[i][j], tempSumX + 1, tempSumY + 1)
			
			tempSumY = tempSumY + math.floor(displayHeight / # def[i])
		end
		
		-- Reinit Values
		tempSumY = -1
		tempSumX = tempSumX + xOffset
	end
end

-- ################### Background #######################
local function background()

	-- myNumSat =  getValueOrDefault("Sats")
	curr_sat = getValueOrDefault("Tmp2")
--	if curr_sat < 100 then
--			myNumSat = curr_sat
--	else 
--			myNumSat = getValueOrDefault("Tmp2") % 1000	-- (xx11 = 11 Satelliten: um die vorderen xx stellen wegzubekommen: geteilt durch 1000 ; der Rest ist im Ergebnis(Modulo)
--			myQualSat = getValueOrDefault("Tmp2") % 1000
--	end 
	
	if (FC_type == "APM") then
		APM_RPM ()
	end
	
end

-- ###################### INIT ##########################
local function init()
-- ########################## CHANGE Here ##########   LIPO for 5 or 7S // FlightController_Type ##########################
-- ##																													 ##

	battype = 0			-- dont Support 5s&7s Battery, its Danger to Detect: if you have an Empty 8s its lock like an 7s...
-- ##					-- set fix to 5S or 7S change "battype = 0" to "battype = 5" 
	FC_type = "APM"		-- possible "BF", "INAV", "APM" ....
	
-- ##																													 ##
-- ########################################################################################################################	

	-- Calculate xOffset depending on how many single column widgets there are
	numSingleCols = 0
	numMultiCols = 0
	myQualSat = 0
	HomeAlt = 0
	set_prearm = false 
	prearmheading = 0
	pilotlat = 0
	pilotlon = 0
	sinCorr = 0
	cosCorr = 0
	offsetX = 0
	offsetY = 0
	radarxtmp = 0
	divtmp = 0
	upppp = 20480
	divvv = 2048 --12 mal teilen
	last_pos_lon = 0
	last_pos_lat = 0
	oldTime={0,0,0,0,0,0}
	Time={0,0,0,0,0,0}
	lastsaynbattpercent=100
	totalbatteryComsum = 0
	compare = 0
	myMaxV = 0
	myMinV = 0
	flightModeNumber = 0
	myNumSat   = 0



	for i=1, # widgetDefinition, 1
	do
		if (# widgetDefinition[i] == 1) then
			numSingleCols = numSingleCols + 1
		else
			numMultiCols = numMultiCols + 1
		end
	end
	xOffsetMulti = (displayWidth - (numSingleCols * xOffsetSingle)) / numMultiCols
	
end

-- ####################### Run #########################
local function run(event)
	
	-- Clear necessary
	lcd.clear()	
	
	-- update current GPS
    background()

	-- Grid
	buildGrid(widgetDefinition)
end

return{init=init,run=run,background=background}